set statistics io on
set nocount on

create table #CongressInformation
(
	Id int IDENTITY(1,1),
	String varchar(500)
);
 
insert into #CongressInformation(String) values
('Bob,Dole,Kansas,Senate,1969,1996'),
('Robert,Taft,Ohio,House,1920,1930'),
('Robert,Taft,Ohio,Senate,1938,1953');
/* Create a more substantial data set.  Run this a few dozen times to give us enough test data (say, ~15000) */
GO 400	

--Make sure we have ~15K records before continuing.
select count(*) from #CongressInformation

/* Set up our comparison tables. */
IF EXISTS(select * from tempdb.sys.objects where name like '#NormalizedCongressInformation_Tally%')
	drop table #NormalizedCongressInformation_Tally;
create table #NormalizedCongressInformation_Tally
(
	Id int,
	FirstName varchar(50),
	LastName varchar(50),
	State varchar(50),
	Branch varchar(50),
	BeginYear int,
	EndYear int
);

IF EXISTS(select * from tempdb.sys.objects where name like '#NormalizedCongressInformation_RBAR%')
	drop table #NormalizedCongressInformation_RBAR
create table #NormalizedCongressInformation_RBAR
(
	Id int,
	FirstName varchar(50),
	LastName varchar(50),
	State varchar(50),
	Branch varchar(50),
	BeginYear int,
	EndYear int
);

GO

/*
	The good way:  use a tally table to split everything.
*/
WITH records as
(
	SELECT
		Id,
		ROW_NUMBER() over (partition by Id order by N) as ColumnNumber,
		SUBSTRING(',' + p.String + ',', N+1, CHARINDEX(',', ',' + p.String + ',', N+1 ) - N-1) as Value
	FROM
		dbo.Tally t
		cross join #CongressInformation p
	WHERE
		N < LEN(',' + p.String + ',')
		AND SUBSTRING(',' + p.String + ',', N, 1) = ','
)
INSERT INTO #NormalizedCongressInformation_Tally
SELECT 
	Id,
	MAX(CASE WHEN ColumnNumber = 1 THEN Value ELSE NULL END),
	MAX(CASE WHEN ColumnNumber = 2 THEN Value ELSE NULL END),
	MAX(CASE WHEN ColumnNumber = 3 THEN Value ELSE NULL END),
	MAX(CASE WHEN ColumnNumber = 4 THEN Value ELSE NULL END),
	MAX(CASE WHEN ColumnNumber = 5 THEN Value ELSE NULL END),
	MAX(CASE WHEN ColumnNumber = 6 THEN Value ELSE NULL END)
FROM
	records
GROUP BY
	Id;

SELECT * FROM #NormalizedCongressInformation_Tally;

GO

/* 
	The bad way:  use a WHILE loop to loop through each result, 
	and then another WHILE loop to loop through the String field. 
*/
DECLARE
	@CurrentID INT = 0,
	@StringInput VARCHAR(500),
	@Value VARCHAR(50),
	@ColumnNumber INT = 0;
DECLARE
	@OutputTable table
	(
		Id int,
		Value varchar(50),
		ColumnNumber int
	);
DECLARE @pDelimiter char(1) = ',';

WHILE EXISTS (SELECT * FROM #CongressInformation WHERE Id > @CurrentID)
BEGIN
	SELECT TOP 1
		@CurrentID = Id,
		@StringInput = String
	FROM
		#CongressInformation
	WHERE
		Id > @CurrentID
	ORDER BY
		Id ASC;

	SET @ColumnNumber = 0;

	WHILE LEN(@StringInput) > 0
	BEGIN
		SET @Value = LEFT(
							@StringInput, 
							ISNULL(NULLIF(CHARINDEX(@pDelimiter, @StringInput) - 1, -1),
							LEN(@StringInput))
						);
		SET @StringInput = SUBSTRING(
										@StringInput,
										ISNULL(NULLIF(CHARINDEX(@pDelimiter, @StringInput), 0),
										LEN(@StringInput)) + 1, LEN(@StringInput)
									);
		SET @ColumnNumber = @ColumnNumber + 1;

		INSERT INTO @OutputTable ( [ID], [Value], [ColumnNumber] )
		VALUES ( @CurrentID, @Value, @ColumnNumber )
	END

END

INSERT INTO #NormalizedCongressInformation_RBAR
SELECT 
	Id,
	MAX(CASE WHEN ColumnNumber = 1 THEN Value ELSE NULL END),
	MAX(CASE WHEN ColumnNumber = 2 THEN Value ELSE NULL END),
	MAX(CASE WHEN ColumnNumber = 3 THEN Value ELSE NULL END),
	MAX(CASE WHEN ColumnNumber = 4 THEN Value ELSE NULL END),
	MAX(CASE WHEN ColumnNumber = 5 THEN Value ELSE NULL END),
	MAX(CASE WHEN ColumnNumber = 6 THEN Value ELSE NULL END)
FROM
	@OutputTable
GROUP BY
	Id;

SELECT * FROM #NormalizedCongressInformation_RBAR;

GO
