set statistics io on;
set statistics time on;

--Date validation
declare @somedates table
(
	adate varchar(8)
);

insert into @somedates(adate) values
('19990501'),
('20111499'),	/* Obviously bad. */
('20090101'),
('20111305'),	/* Bad...or is it? */
('ababyyyy'),	/* Obviously bad. */
('9901'),		/* Should be bad. */
('Jun 1 09'),	/* Should be bad. */
('20030419');

--select adate, isdate(adate) from @somedates;

DECLARE @StartDate date = '1991-01-01';
DECLARE @EndDate date = '2015-01-01';

--The physical table version.
with tallyDates as
(
	SELECT 
		CONVERT(varchar(8), DATEADD(d, N, @StartDate), 112) as ValidDate
	FROM 
		dbo.Tally
	WHERE 
		DATEADD(d, N, @StartDate) <= @EndDate
)
SELECT 
	sd.*
FROM 
	@somedates sd
	left outer join tallyDates td on sd.adate = td.ValidDate
WHERE 
	td.ValidDate is null;
	
--The CTE version.	
WITH E1(N) AS 
(
	SELECT 1 UNION ALL SELECT 1 UNION ALL SELECT 1 UNION ALL 
	SELECT 1 UNION ALL SELECT 1 UNION ALL SELECT 1 UNION ALL 
	SELECT 1 UNION ALL SELECT 1 UNION ALL SELECT 1 UNION ALL SELECT 1
),										--10E+1 or 10 rows
E2(N) AS (SELECT 1 FROM E1 a, E1 b),	--10E+2 or 100 rows
E4(N) AS (SELECT 1 FROM E2 a, E2 b),	--10E+4 or 10,000 rows max
cteTally(N) AS 
(
     SELECT 0 UNION ALL
     SELECT TOP (DATEDIFF(dd, @StartDate, @EndDate)) 
					ROW_NUMBER() OVER (ORDER BY (SELECT NULL)) 
		FROM E4
),
tallyDates as
(
	SELECT 
		CAST(YEAR(dateadd(d,n,@StartDate)) AS char(4)) +
		RIGHT('0' + CAST(MONTH(dateadd(d,n,@StartDate)) AS varchar(2)),2) +
		RIGHT('0' + CAST(DAY(dateadd(d,n,@StartDate)) AS varchar(2)),2) as ValidDate
	FROM 
		cteTally
	WHERE 
		DATEADD(d,n,@StartDate) <= @EndDate
)
SELECT 
	sd.*
FROM 
	@somedates sd
	left outer join tallyDates td on sd.adate = td.ValidDate
WHERE 
	td.ValidDate is null;
	
--Why not just use ISDATE() here?
select ISDATE('19990501');
select ISDATE('20111499');
select ISDATE('9901');
select ISDATE('Jun 1 09');	